
import logging
import os


class log():
    # create logger
    logger = logging.getLogger('silverpeak-prometheus')
    logger.setLevel(logging.DEBUG)
    
    # create console handler and set level to debug
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    
    # create formatter
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    
    # add formatter to ch
    ch.setFormatter(formatter)
    
    # add ch to logger
    logger.addHandler(ch)

    def __init__(self):
        pass
    def debug(self,message):
        self.logger.debug(message)
    def info(self,message):
        self.logger.info(message)
    def error(self,message):
        self.logger.error(message)


class logToFile():
    # Setting directory for debug files
    pathDir = os.path.dirname(os.path.realpath(__file__))
    debugFile = os.path.join(pathDir, "debug/debug.log")

    def __init__(self):
        pass

    def debug(self,message,debug=False):
        if debug == True:
             logging.basicConfig(filename=self.debugFile, 
             encoding='utf-8', 
             format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
             level=logging.DEBUG)
             logging.debug(message)



import threading
from prometheus_client import start_http_server
from .logger import log
from .file import settings
from .utls import getApplianceID
from .collectors import collectOrchestratorMetrics,getAllAppliances,applianceCollector,collectApplicationMetrics
from time import sleep as s


def main():
# Loads the application version and logs it
#    appVersion = getAppVersion()
#    log().info(f'starting application on version exporter {appVersion}')
# Loading user inputs via CLI
    fileConfig = settings()
# Setting common used varaibles
    prometheusPort = fileConfig.port()['port']
    orchConfig = fileConfig.orch()
    silverpeakUrl = orchConfig["url"]
    silverpeakKey = orchConfig['key']
    silverpeakSSL = orchConfig['verify_ssl']
    metricsOrchestratorInterval = fileConfig.metricsOrchestrator()['interval']
    debug = fileConfig.debug()['debug']
    Break = fileConfig.Break()['Break']

# Start up the server to expose the metrics.
    start_http_server(prometheusPort)
    log().info(f'starting prometheus exporter on {prometheusPort}')
# Start the thread to collect application metrics.
    t = threading.Thread(target=collectApplicationMetrics, args=(
            debug,
            Break,
        ))
    log().info('starting collection of application metrics')
    t.start()   

    

# Triggering the thread that will collect orchestrator metrics
    if fileConfig.metricsOrchestrator()['collect']:
        log().info(f'starting orchestrator metric collection on {silverpeakUrl}')
        log().info(f'starting orchestrator metric collection every {metricsOrchestratorInterval} seconds ')
        x = threading.Thread(target=collectOrchestratorMetrics, args=(
            silverpeakUrl,
            silverpeakKey,
            silverpeakSSL,
            metricsOrchestratorInterval,
            debug,
            Break,
        ))
        x.start()   
        s(2)     
    else:
        log().info(f'skipping orchestrator metric collection')    

# Triggering the thread that will collect appliance metrics
    if fileConfig.metricsAppliance()['appliances'] != False:
        appliances = getAllAppliances(silverpeakUrl,silverpeakSSL,silverpeakKey)
    
        for key in fileConfig.metricsAppliance()['appliances'].keys():
            applianceName = key
            applianceID = getApplianceID(name=applianceName,applianceDict=appliances)
    
            for feature in fileConfig.metricsAppliance()['appliances'][key]:
                z = threading.Thread(target=applianceCollector, kwargs={
                    'feature' : feature,
                    'url' : silverpeakUrl,
                    'ne_pk' : applianceID,
                    'applianceName' : applianceName,
                    'key' : silverpeakKey,
                    'verify_ssl' : silverpeakSSL,
                    'debug' : debug,
                    'Break' : Break,
                })
                z.start()
            s(2)
    else:
        log().info(f'skipping appliance metric collection')    


if __name__ == '__main__':
    main()


import argparse
from .logger import log

def parser():
    parser = argparse.ArgumentParser()
##############################################################################    
#                            Exporter CLI Inputs
##############################################################################
    parser.add_argument('-d', '--debug',
                                  required=False,
                                  #choices='ec',
                                  action='store_true',
                                  help="debug the cli")
    parser.add_argument('-v', '--version',
                                  required=False,
                                  #choices='ec',
                                  action='store_true',
                                  help="shows cli tool version")

    parser.add_argument('-p', '--port',
                                  required=False,
                                  type=int,
                                  nargs="?",
                                  help="port for prometheus exporter")
    parser.add_argument('-o', '--orch',
                                  required=False,
                                  nargs="?",
                                  help="Silverpeak orchestrator url")

    parser.add_argument('-k', '--key',
                                  required=False,
                                  nargs="?",
                                  help="Silverpeak orchestrator api key")
    parser.add_argument('-f', '--file-path',
                                  required=False,
                                  nargs="?",
                                  help="Configuration file path")
    parser.add_argument('-b', '--Break',
                                  required=False,
                                  #choices='ec',
                                  action='store_true',
                                  help="run once or infinite")

    options = parser.parse_args()    # This is the return of the function and what is passed to other functions
    outOptions = parser.parse_args() # This are the input that will be printed to the screen if debug flag is enable


    if (outOptions.debug == True):
        del outOptions.key
        log().info(outOptions)

    if (options.version == True):
        from importlib_metadata import version
        print(f'version...installed:',version('silverpeak-exporter'))

    return options
    


#Importing prometheus library
from prometheus_client import Info,Gauge,Enum

#---------#---------#---------#---------#---------#
# Application Metrics
#---------#---------#---------#---------#---------#
activeThreads = Gauge('activeThreads', 'number of total python threads')

#---------#---------#---------#---------#---------#
# Orchestrator Metrics
#---------#---------#---------#---------#---------#

uptime = Info('uptime', 'silverpeak orchestrator uptime',['orchName'])
release = Info('release', 'silverpeak orchestrator release version',['orchName'])
platform = Info('platform', 'silverpeak orchestrator platform',['orchName'])
numActiveUsers = Gauge('numActiveUsers', 'number of active users logged in',['orchName'])
activeAlarms = Gauge('activeAlarms', 'summary of active alarms for Orchestrator and all appliances',['orchName','severity'])
totalAppliances = Gauge('totalAppliances', 'number of total appliances in your orchestrator',['orchName'])
licenseNumMini  = Gauge('licenseNumMini', 'number of mini licenses',['orchName'])
licenseNumBase  = Gauge('licenseNumBase', 'number of base licenses',['orchName'])
licenseNumBoost  = Gauge('licenseNumBoost', 'number of boost licenses',['orchName'])
licenseNumPlus  = Gauge('licenseNumPlus', 'number of plus licenses',['orchName'])
licenseNumTier20M = Gauge('licenseNumTier20M', 'number of 20M licenses',['orchName'])
licenseNumTier50M = Gauge('licenseNumTier50M', 'number of 50M licenses',['orchName'])
licenseNumTier100M = Gauge('licenseNumTier100M', 'number of 100M licenses',['orchName'])
licenseNumTier200M = Gauge('licenseNumTier200M', 'number of 200M licenses',['orchName'])
licenseNumTier500M = Gauge('licenseNumTier500M', 'number of 500M licenses',['orchName'])
licenseNumTier1G = Gauge('licenseNumTier1G', 'number of 1G licenses',['orchName'])
licenseNumTier2G = Gauge('licenseNumTier2G', 'number of 2G licenses',['orchName'])
licenseNumTierUM = Gauge('licenseNumTierUM', 'number of Unlimited licenses',['orchName'])
totalOverlays = Gauge('totalOverlays', 'number of overlays configured',['orchName'])
totalRegions = Gauge('totalRegions', 'number of regions configured',['orchName'])
totalSaasApps = Gauge('totalSaasApps', 'list of unique SaaS applications in the internet services database',['orchName'])
totalAppDefinition = Gauge('totalAppDefinition', 'count for application definition data for Address Map from Cloud Portal',['orchName'])
totalOrchSaasApps = Gauge('totalOrchSaasApps', 'count of internet services defined on Orchestrator',['orchName'])
orchPortalStatus = Enum('orchPortalStatus', 'current connectivity status between Orchestrator and Cloud Portal',['orchName'], states=['unable to connect', 'connected', 'connecting'])
cloudPortalServices = Gauge('cloudPortalServices', 'silverpeak cloud portal service and status',['orchName','portalService','status'])
totalTunnelCount = Gauge('totalTunnelCount', 'get total tunnel count across all appliances',['orchName'])
totalCPUs = Gauge('totalCPUs', 'the total amount of CPU on the orchestrator, works only when orch is not running in container',['orchName'])
loadAverage = Gauge('loadAverage', 'orchestrator load average, works only when orch is not running in container',['orchName'])
totalMemSize = Gauge('totalMemSize', 'total memory of the server, works only when orch is not running in container',['orchName'])
maintenaceModeAppliances = Gauge('maintenacModeAppliances', 'appliances currently in maintenance mode',['orchName'])
#---------#---------#---------#---------#---------#
# Appliance Metrics
#---------#---------#---------#---------#---------#
applianceHostname = Info('applianceHostname', 'name of the silverpeak edge connect',['applianceName'])
applianceModel = Info('applianceModel', 'mode of the silverpeak appliance',['applianceName'])
appliancePlatform = Info('appliancePlatform', 'platform of the silverpeak appliance',['applianceName'])
applianceUptime = Gauge('applianceUptime', 'uptime in milliseconds for the silverpeak appliance',['applianceName'])
applianceUptimeString = Info('applianceUptimeString', 'uptime in string format for the silverpeak appliance',['applianceName'])
applianceRelease = Info('applianceRelease', 'appliance release version',['applianceName'])
applianceDeploymentMode = Info('applianceDeploymentMode', 'the deployment mode of the appliance',['applianceName'])
applianceAlarm = Gauge('applianceAlarm', 'summary of active alarms for given appliance',['applianceName','severity'])
applianceCPU = Gauge('applianceCPU', 'cpu utilization in porcentage for each appliance core',['applianceName','cpu_core','metric'])
applianceDiskUsage = Gauge('applianceDiskUsage', 'appliance disk utilization for each mount point',['applianceName','mount','metric'])
applianceRebootRequiered = Enum('applianceRebootRequiered', 'check with appliance if a reboot is required from changes',['applianceName'], states=['True', 'False'])
applianceMemory = Gauge('applianceMemory', 'get appliance memory related information',['applianceName','metric'])

#---------#---------#---------#---------#---------#
# BGP Metrics
#---------#---------#---------#---------#---------#
bgpNumVrfs = Gauge('bgpNumVrfs', 'total number of vrf using bgp',['applianceName',])
bgpProcessState = Info('bgpProcessState', 'state of the bgp process',['applianceName'])
bgpRejectMismatches = Gauge('bgpRejectMismatches', 'bgp neighbors rejected by a mismatch',['applianceName',])
bgpNumPeers = Gauge('bgpNumPeers', 'total number of bgp peers',['applianceName',])
bgpSubsInstalled = Gauge('bgpSubsInstalled', 'total numbers of subnets installed in the bgp process',['applianceName',])
bgpOspfRoutes = Gauge('bgpOspfRoutes', 'total number of routes originated by OSPF',['applianceName',])
bgpIBGPRoutes = Gauge('bgpIBGPRoutes', 'total number of routes originated by iBGP',['applianceName',])
bgpEBGPRoutes = Gauge('bgpEBGPRoutes', 'total number of routes originated by eBGP',['applianceName',])
bgpNeighborState = Gauge('bgpNeighborState', 'state of bgp neighbor',['applianceName','peer_ip'])
bgpNeighborStateStr = Info('bgpNeighborStateStr', 'state of bgp neighbor in string',['applianceName','peer_ip','asn'])
bgpNeighborUptime = Gauge('bgpNeighborUptime', 'uptime of the bgp neighbor',['applianceName','peer_ip'])
bgpNeighborReceivedPrefix = Gauge('bgpNeighborReceivedPrefix', 'total number of prefixes received from the neighbor',['applianceName','peer_ip'])
bgpNeighborSentPrefix = Gauge('bgpNeighborSentPrefix', 'total number of prefixes advertised to the neighbor',['applianceName','peer_ip'])



#---------#---------#---------#---------#---------#
# Flows Metrics
#---------#---------#---------#---------#---------#

activeTotalFlows = Gauge('activeTotalFlows', 'number of active flows',['applianceName',])
activeStaleFlows = Gauge('activeStaleFlows', 'number of active stale flows',['applianceName',])
activeInconsistentFlows = Gauge('activeInconsistentFlows', 'number of active inconsistent flows',['applianceName',])
activeFlowsWithIssues = Gauge('activeFlowsWithIssues', 'number of active flows with issues',['applianceName',])
activeFlowsOptimized = Gauge('activeFlowsOptimized', 'number of active optimized flows',['applianceName',])
activeFlowsWithIgnores = Gauge('activeFlowsWithIgnores', 'no description in swagger',['applianceName',])
activeFlowsPassthrough = Gauge('activeFlowsPassthrough', 'number of active passthrough flows',['applianceName',])
activeFlowsManagement = Gauge('activeFlowsManagement', 'number of active management flows',['applianceName',])
activeFlowsAsymmetric = Gauge('activeFlowsAsymmetric', 'number of active asymmetric flows',['applianceName',])
activeFlowsRouteDropped = Gauge('activeFlowsRouteDropped', 'number of active flows dropped due to route',['applianceName',])
activeFlowsFirewallDropped = Gauge('activeFlowsFirewallDropped', 'number of active flows dropped due to firewall',['applianceName',])
inactiveTotalFlows = Gauge('inactiveTotalFlows', 'number of inactive flows',['applianceName',])
inactiveStaleFlows = Gauge('inactiveStaleFlows', 'number of inactive stale flows',['applianceName',])
inactiveInconsistentFlows = Gauge('inactiveInconsistentFlows', 'number of inactive inconsistent flows',['applianceName',])
inactiveFlowsWithIssues = Gauge('inactiveFlowsWithIssues', 'number of inactive flows with issues',['applianceName',])
inactiveFlowsOptimized = Gauge('inactiveFlowsOptimized', 'number of inactive optimized flows',['applianceName',])
inactiveFlowsWithIgnores = Gauge('inactiveFlowsWithIgnores', 'no description in swagger',['applianceName',])
inactiveFlowsPassthrough = Gauge('inactiveFlowsPassthrough', 'number of inactive passthrough flows',['applianceName',])
inactiveFlowsManagement = Gauge('inactiveFlowsManagement', 'number of inactive management flows',['applianceName',])
inactiveFlowsAsymmetric = Gauge('inactiveFlowsAsymmetric', 'number of inactive asymmetric flows',['applianceName',])
inactiveFlowsRouteDropped = Gauge('inactiveFlowsRouteDropped', 'number of inactive flows dropped due to route',['applianceName',])
inactiveFlowsFirewallDropped = Gauge('inactiveFlowsFirewallDropped', 'number of inactive flows dropped due to firewall',['applianceName',])


import os
from .logger import log
import tomli
import yaml

# Creates the debug folder to save information
def createDebugFolder(debug:bool=False):
    if debug == True:
        log().info("starting program in debug mode")
        pathDir = os.path.dirname(os.path.realpath(__file__))
        listOfFolders = os.listdir(pathDir)
        debugFolder = os.path.join(pathDir, "debug")
        debugFile = os.path.join(debugFolder,'debug.log')
        resultFile = os.path.join(debugFolder,'result.yml')
        if 'debug' in listOfFolders:
            log().info(f"creating new debug.log file on {debugFolder}")
            file = open(debugFile, 'w+')
            file.close()
            log().info(f"creating new result.yml file on {debugFolder}")
            file = open(resultFile, 'w+')
            file.close()
        else:
            try:
                log().info(f"creating debug folder to store files at {debugFolder}")
                os.mkdir(debugFolder)
                file = open(debugFile, 'w+')
                file.close()
                file = open(resultFile, 'w+')
                file.close()
            except Exception as error:
                log().error(error)
    else:
        pass

# Confirms the appliance returned a 200 and not an error
def confirmReturn(func , dictionary:dict, debug:bool):
    if debug == True:
        try:
            if dictionary['status_code'] != 200:
                log().error({func : dictionary})
        except:
            pass
    else:
        pass

def writeResult(func, result, debug):
    if debug:
        pathDir = os.path.dirname(os.path.realpath(__file__))
        debugFolder = os.path.join(pathDir, "debug")
        resultFile = os.path.join(debugFolder,'result.yml')
    
        with open(resultFile, "a") as ymlFile:
            data = { func : result}
            yaml.dump(data, ymlFile)


# Wrapper to except errors
def errorHandler(func):
    def wrapper(*args):
        try:
            out = func(*args)
            result = True
        except Exception as error:
            log().error({func : error})
            out = error
            result = False
        return {func.__name__ : out,
                'result' : result}
    return wrapper


# Returns the function that needs to be called for a given feature
def checkCollector(feature,collectorMap):
    for f,s in collectorMap.items():
        for k in feature.keys():
            if f == k:
                return s

           
# Returns the NE_PK of the appliance base on the name
def getApplianceID(name,applianceDict:dict):
    try:
        applianceID = applianceDict.get(name)
    except Exception as error:
        log().error(f"error getting the appliance ID for {name}")
        log().error(error)
    return applianceID


# Reads the toml file to find out application version
#def getAppVersion():
#    pathDir = os.path.dirname(os.path.realpath("pyproject.toml"))
#    tomlFile = os.path.join(pathDir,'pyproject.toml')
#    with open(tomlFile, "rb") as f:
#        data = tomli.load(f)
#        return data['tool']['poetry']['version']


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['silverpeak_exporter']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML==6.0',
 'prometheus-client==0.15.0',
 'pyedgeconnect==0.15.3a1.dev0',
 'requests==2.28.1',
 'tomli==2.0.1']

entry_points = \
{'console_scripts': ['spexporter = silverpeak_exporter.main:main']}

setup_kwargs = {
    'name': 'silverpeak-exporter',
    'version': '0.1.8',
    'description': 'Prometheus exporter for Silverpeak SD-WAN Appliances.',
    'long_description': '![Release](https://img.shields.io/github/v/tag/ipHeaders/silverpeak-prometheus)\n![License](https://img.shields.io/github/license/ipHeaders/silverpeak-prometheus)\n![Open Issues](https://img.shields.io/github/issues/ipHeaders/silverpeak-prometheus)\n![](https://img.shields.io/github/languages/top/ipHeaders/silverpeak-prometheus)\n![](https://img.shields.io/pypi/pyversions/silverpeak-exporter)\n![Docker Pulls](https://img.shields.io/docker/pulls/ipheaders/silverpeak-prometheus)\n[![Downloads](https://static.pepy.tech/personalized-badge/silverpeak-exporter?period=total&units=none&left_color=grey&right_color=orange&left_text=PyPI%20Downloads)](https://pepy.tech/project/silverpeak-exporter)\n\n# silverpeak-prometheus-exporter\nSilverpeak/Aruba SD-WAN Prometheus Exporter, this tool is to query the Silverpeak/Aruba SD-WAN orchestrator export the metrics to a prometheus database. \n\n```diff\n! For better security practices make sure the API key is read only.\n```\n\n## Requierements\n\n- Orchestraor API Key\n- Python3.9>=\n\n## Installation Methods\n- Installing using [Pypi](https://github.com/ipHeaders/silverpeak-prometheus/tree/main/docs/installing_using_pypi.md)\n- Installing directly from [Github](https://github.com/ipHeaders/silverpeak-prometheus/tree/main/docs/installing_from_github.md)\n- Running on [Container](https://github.com/ipHeaders/silverpeak-prometheus/tree/main/docs/running_on_container.md)\n\n## References\n- Avaiable Exposed Metrics [Metrics](https://github.com/ipHeaders/silverpeak-prometheus/tree/main/docs/metrics.md)\n- DockerHub Project [Docker](https://hub.docker.com/r/ipheaders/silverpeak-prometheus)\n- Grafana Dashboard [Grafana](https://grafana.com/grafana/dashboards/17745-spexporter/)\n- Application Flow [Flow](https://github.com/ipHeaders/silverpeak-prometheus/tree/main/docs/appflow.png)\n\n## Grafana Example\n### Orchestrator\n![alt text](docs/grafana_orch.png)\n\n### Appliances\n![alt text](docs/grafana_appliace.png)\n\n### BGP\n![alt text](docs/grafana_bgp.png)\n\n## Maintainer\n[IPheaders](https://github.com/ipHeaders)\n',
    'author': 'IP Headers',
    'author_email': 'ipHeaders@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import inspect
from pyedgeconnect import Orchestrator
from time import sleep as wait
from .logger import log,logToFile
from .utls import confirmReturn,errorHandler,writeResult
from .metrics import *
import threading

#--------------------------------------------------#
# Application Metrics Collections
#--------------------------------------------------#

class collectApplicationMetrics():
    def __init__(self,debug, Break):
        self.debug = debug
        self.Break = Break
        while True:
            # get a list of all the methods in this class
            # loops over the list starting at index 1 to bypass __init__
            # calls the method
            methodList = inspect.getmembers(self, predicate=inspect.ismethod)
            for m in range(1, len(methodList)):
                i = methodList[m][1]()

                logToFile().debug(message=dict({methodList[m][1].__name__ : i}), debug=self.debug) 
                confirmReturn(func=methodList[m][1].__name__ ,dictionary=i, debug=self.debug) 
                writeResult(func=list(i.keys())[0], result=i['result'], debug=self.debug)
            if self.Break == False:
                wait(60)
            else:
                break


    @errorHandler
    def _getactiveThreads(self):
        activeThread = threading.active_count()
        sysOut = {"activeThreads" : activeThread}
        activeThreads.set(activeThread)
        return sysOut
       

#--------------------------------------------------#
# Metrics Collections for the Orchestrator
#--------------------------------------------------#
class collectOrchestratorMetrics():
    def __init__(self,url,key,verify_ssl,interval,debug, Break):
        self.url = url
        self.key = key
        self.verify_ssl = verify_ssl
        self.interval = interval
        self.debug = debug
        self.Break = Break
        self.orch = Orchestrator(url=self.url, verify_ssl=self.verify_ssl, api_key=self.key )
        self.hostname = self.orch.get_orchestrator_hostname()['gms_hostname']
        while True:
            # get a list of all the methods in this class
            # loops over the list starting at index 1 to bypass __init__
            # calls the method
            methodList = inspect.getmembers(self, predicate=inspect.ismethod)
            for m in range(1, len(methodList)):
                i = methodList[m][1]()

                logToFile().debug(message=dict({methodList[m][1].__name__ : i}), debug=self.debug) 
                confirmReturn(func=methodList[m][1].__name__ ,dictionary=i, debug=self.debug) 
                writeResult(func=list(i.keys())[0], result=i['result'], debug=self.debug)
            if self.Break == False:
                wait(self.interval)
            else:
                break

    @errorHandler
    def _getOrchServerBrief(self):
       orch_return = self.orch.get_orchestrator_server_info()   
       uptime.labels(orchName=self.hostname).info({'uptime' : orch_return['uptime']}) #Setting Metric
       release.labels(orchName=self.hostname).info({'release' : orch_return['release']}) #Setting Metric
       platform.labels(orchName=self.hostname).info({'platform' : orch_return['platform']}) #Setting Metric
       numActiveUsers.labels(orchName=self.hostname).set(orch_return['numActiveUsers']) #Setting Metric

       if orch_return['platform'] != "DOCKER":
           totalCPUs.labels(orchName=self.hostname).set(orch_return['numCpus']) #Setting Metric
           loadAverage.labels(orchName=self.hostname).set(orch_return['loadAverage']) #Setting Metric
           totalMemSize.labels(orchName=self.hostname).set(orch_return['memSize']) #Setting Metric
       
       return orch_return

    @errorHandler
    def _getAlarmCountOrchestratoAndAppliances(self):
        orch_return = self.orch.get_alarm_count_orchestrator_and_appliances() 
        for key,value in orch_return.items():
            activeAlarms.labels(orchName=self.hostname,severity=key).set(value) #Setting Metric
        return orch_return                

    @errorHandler
    def _getTotalAppliances(self):
        orch_return = self.orch.get_appliances() 
        totalAppliances.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return


    @errorHandler
    def _getLicenseState(self):
        orch_return = self.orch.get_portal_licensed_summary() 
        licenseState = orch_return['licenseState']['fx']
        
        licenseNumMini.labels(orchName=self.hostname).set(licenseState['numMini']) #Setting Metric
        licenseNumBase.labels(orchName=self.hostname).set(licenseState['numBase']) #Setting Metric
        licenseNumBoost.labels(orchName=self.hostname).set(licenseState['numBoost']) #Setting Metric
        licenseNumPlus.labels(orchName=self.hostname).set(licenseState['numPlus']) #Setting Metric
        numTier = {
            "20000" : licenseNumTier20M,
            "50000" : licenseNumTier50M,
            "100000" : licenseNumTier100M,
            "200000" : licenseNumTier200M,
            "500000" : licenseNumTier500M,
            "1000000" : licenseNumTier1G,
            "2000000" : licenseNumTier2G,
            "1000000000" : licenseNumTierUM,
        }
        for key,value in numTier.items():
            if key in licenseState['numTier']:
                value.labels(orchName=self.hostname).set(licenseState['numTier'][key]) #Setting Metric
            else:
                value.labels(orchName=self.hostname).set(0) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalOverlays(self):
        orch_return = self.orch.get_all_overlays_config() 
        totalOverlays.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalRegions(self):
        orch_return = self.orch.get_all_regions() 
        totalRegions.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalSaasApps(self):
        orch_return = self.orch.get_all_saas_apps() 
        totalSaasApps.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalAppDefinition(self):
        orch_return = self.orch.get_app_definition_total() 
        totalAppDefinition.labels(orchName=self.hostname).set(orch_return) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalOrchSaasApps(self):
        orch_return = self.orch.get_count_of_saas_apps() 
        totalOrchSaasApps.labels(orchName=self.hostname).set(orch_return['count']) #Setting Metric
        return orch_return

    @errorHandler
    def _getOrchPortalStatus(self):
        orch_return = self.orch.get_orchestrator_to_cloud_portal_status()
        mapping = {
            0 : 'unable to connect',
            1 : 'connected',
            2 : 'connecting'
        }
        orchPortalStatus.labels(orchName=self.hostname).state(mapping[orch_return['portal']]) #Setting Metric
        return orch_return
            
    @errorHandler
    def _getCloudPortalServices(self):
        orch_return = self.orch.get_portal_services_status() 
        for key,value in orch_return.items():
            cloudPortalServices.labels(orchName=self.hostname,portalService=key, status='ok').set(orch_return[key]['state']['ok'])
            cloudPortalServices.labels(orchName=self.hostname,portalService=key, status='fail').set(orch_return[key]['state']['fail'])

        return orch_return

    @errorHandler
    def _getTotalTunnelCount(self):
        orch_return = self.orch.get_total_tunnel_count()
        totalTunnelCount.labels(orchName=self.hostname).set(orch_return['totalTunnelCount'])
        return orch_return

    @errorHandler
    def _getAppliancesInMaintenance(self):
        orch_return = self.orch.get_maintenance_mode_appliances()
        supressAlarm = orch_return['suppressAlarm']
        supressOrch = orch_return['pauseOrchestration']
        
        unique = supressAlarm + list(set(supressOrch) - set(supressAlarm))
        maintenaceModeAppliances.labels(orchName=self.hostname).set(len(unique))
        return orch_return
    
#--------------------------------------------------#
# Metrics Collections for the Appliances
#--------------------------------------------------#

def applianceCollector(**kwargs):
    applianceName = kwargs['applianceName']
    ne_pk = kwargs['ne_pk']
    interval = kwargs['feature']['interval']

    if 'system' in kwargs['feature']:
        try:
            log().info(f'starting system metric collection on {applianceName,ne_pk} every {interval} seconds')

            collectApplianceSystem(
            url = kwargs['url'],
            ne_pk = ne_pk,
            applianceName = applianceName,
            key = kwargs['key'],
            verify_ssl = kwargs['verify_ssl'],
            interval = interval,
            debug = kwargs['debug'],
            Break = kwargs['Break'],
            )
        except Exception as error:
            log().error(f'failed to set argument {error}')

    if 'bgp' in kwargs['feature']:
        try:
            log().info(f'starting bgp metric collection on {applianceName,ne_pk} every {interval} seconds')

            collectApplianceBGP(
            url = kwargs['url'],
            ne_pk = ne_pk,
            applianceName = applianceName,
            key = kwargs['key'],
            verify_ssl = kwargs['verify_ssl'],
            interval = interval,
            debug = kwargs['debug'],
            Break = kwargs['Break'],
            )
        except Exception as error:
            log().error(f'failed to set argument {error}')

    if 'flows' in kwargs['feature']:
        try:
            log().info(f'starting flows metric collection on {applianceName,ne_pk} every {interval} seconds')

            collectApplianceFlows(
            url = kwargs['url'],
            ne_pk = ne_pk,
            applianceName = applianceName,
            key = kwargs['key'],
            verify_ssl = kwargs['verify_ssl'],
            interval = interval,
            debug = kwargs['debug'],
            Break = kwargs['Break'],
            )
        except Exception as error:
            log().error(f'failed to set argument {error}')

def getAllAppliances(url,verify_ssl,api_key):
    try:
        #
        log().info('getting appliances NE_PK')
        sp_dict = {}
        orch = Orchestrator(url=url, verify_ssl=verify_ssl, api_key=api_key)
        orch_return = orch.get_appliances()
        for device in orch_return:
            sp_dict.update({device['hostName']: device['id']})
    except Exception as error:
        log().error(f'failed to get appliances NE_PK {error}')

    return sp_dict



class collectApplianceSystem():
    def __init__(self,url,ne_pk,applianceName,key,verify_ssl,interval,debug, Break):
        self.url = url
        self.ne_pk = ne_pk
        self.applianceName = applianceName
        self.key = key
        self.verify_ssl = verify_ssl
        self.interval = interval
        self.debug = debug
        self.Break = Break
        self.orch = Orchestrator(url=self.url, verify_ssl=self.verify_ssl, api_key=self.key )
        while True:
            # get a list of all the methods in this class
            # loops over the list starting at index 1 to bypass __init__
            # calls the method
            methodList = inspect.getmembers(self, predicate=inspect.ismethod)
            for m in range(1, len(methodList)):
                i = methodList[m][1]()
                logToFile().debug(message=dict({methodList[m][1].__name__ : i}), debug=self.debug) 
                confirmReturn(func=methodList[m][1].__name__ ,dictionary=i, debug=self.debug)
                writeResult(func=list(i.keys())[0], result=i['result'], debug=self.debug)
            if self.Break == False:
                wait(self.interval)
            else:
                break

    @errorHandler
    def _getApplianceAlarms(self):
        path = '/alarm'
        orch_return = self.orch.appliance_get_api(self.ne_pk, path) 
        for key,value in orch_return['summary'].items():
            applianceAlarm.labels(applianceName=self.applianceName,severity=key).set(value)
        return orch_return

    @errorHandler
    def _getApplianceCPU(self):
        path = '/cpustat'
        orch_return = self.orch.appliance_get_api(self.ne_pk, path) 
        latestTime = orch_return['latestTimestamp']
        lastStats = orch_return['data'][-1]
        cpuStats = lastStats[str(latestTime)]

        for cpu in cpuStats:
            cpu_name = cpu['cpu_number']
            del cpu['cpu_number']
            for k,v in cpu.items():
                applianceCPU.labels(applianceName=self.applianceName,cpu_core=cpu_name,metric=k).set(v)

        return orch_return

    @errorHandler
    def _getApplianceDiskUsage(self):
        path = '/diskUsage'
        orch_return = self.orch.appliance_get_api(self.ne_pk, path) 
        for k,v in orch_return.items():
            mount = k
            del v['filesystem']
            for a,b in v.items():
                applianceDiskUsage.labels(applianceName=self.applianceName,mount=mount,metric=a).set(b)
        return orch_return

    @errorHandler
    def _getApplianceRebootRequiered(self):
        path = '/license/isRebootRequired'
        orch_return = self.orch.appliance_get_api(self.ne_pk, path)
        applianceRebootRequiered.labels(applianceName=self.applianceName).state(str(orch_return['isRebootRequired'])) #Setting Metric
        return orch_return

    @errorHandler
    def _getApplianceMemory(self):
        path = '/memory'
        orch_return = self.orch.appliance_get_api(self.ne_pk, path)
        for k,v in orch_return.items():
            applianceMemory.labels(applianceName=self.applianceName,metric=k).set(v)
        return orch_return

    @errorHandler
    def _getApplianceSystemInfo(self):
        path = '/systemInfo'
        orch_return = self.orch.appliance_get_api(self.ne_pk, path)
        try:
            applianceHostname.labels(applianceName=self.applianceName).info({'hostName' : orch_return['hostName']}) #Setting Metric
        except:
            applianceHostname.labels(applianceName=self.applianceName).info({'hostName' : 'null'}) #Setting Metric
        try:
            applianceModel.labels(applianceName=self.applianceName).info({'modelShort' : orch_return['modelShort']}) #Setting Metric
        except:
            applianceModel.labels(applianceName=self.applianceName).info({'modelShort' : 'null'}) #Setting Metric
        try:
            appliancePlatform.labels(applianceName=self.applianceName).info({'platform' : orch_return['platform']}) #Setting Metric
        except:
            appliancePlatform.labels(applianceName=self.applianceName).info({'platform' : 'null'}) #Setting Metric
        try:
            applianceUptime.labels(applianceName=self.applianceName).set(orch_return['uptime']) #Setting Metric
        except:
            applianceUptime.labels(applianceName=self.applianceName).set(0) #Setting Metric
        try:
            applianceUptimeString.labels(applianceName=self.applianceName).info({'uptimeString' : orch_return['uptimeString']}) #Setting Metric
        except:
            applianceUptimeString.labels(applianceName=self.applianceName).info({'uptimeString' : 'null'}) #Setting Metric
        try:
            applianceRelease.labels(applianceName=self.applianceName).info({'release' : orch_return['release']}) #Setting Metric
        except:
            applianceRelease.labels(applianceName=self.applianceName).info({'release' : 'null'}) #Setting Metric
        try:
            applianceDeploymentMode.labels(applianceName=self.applianceName).info({'deploymentMode' : orch_return['deploymentMode']}) #Setting Metric
        except:
            applianceDeploymentMode.labels(applianceName=self.applianceName).info({'deploymentMode' : 'null'}) #Setting Metric
            
        return orch_return

class collectApplianceBGP():
    def __init__(self,url,ne_pk,applianceName,key,verify_ssl,interval,debug, Break):
        self.url = url
        self.ne_pk = ne_pk
        self.applianceName = applianceName
        self.key = key
        self.verify_ssl = verify_ssl
        self.interval = interval
        self.debug = debug
        self.Break = Break
        self.orch = Orchestrator(url=self.url, verify_ssl=self.verify_ssl, api_key=self.key )
        while True:
            # get a list of all the methods in this class
            # loops over the list starting at index 1 to bypass __init__
            # calls the method
            methodList = inspect.getmembers(self, predicate=inspect.ismethod)
            for m in range(1, len(methodList)):
                i = methodList[m][1]()

                logToFile().debug(message=dict({methodList[m][1].__name__ : i}), debug=self.debug) 
                confirmReturn(func=methodList[m][1].__name__ ,dictionary=i, debug=self.debug) 
                writeResult(func=list(i.keys())[0], result=i['result'], debug=self.debug)
            if self.Break == False:
                wait(self.interval)
            else:
                break

    @errorHandler
    def _getApplianBGPState(self):
        orch_return = self.orch.get_appliance_bgp_state(self.ne_pk)
        bgpNumVrfs.labels(applianceName=self.applianceName).set(orch_return['summary']['num_vrfs']) #Setting Metric
        bgpProcessState.labels(applianceName=self.applianceName).info({'bgpProcessState' : orch_return['summary']['bgp_state_str']}) #Setting Metric
        bgpRejectMismatches.labels(applianceName=self.applianceName).set(orch_return['summary']['reject_mismatches']) #Setting Metric
        bgpNumPeers.labels(applianceName=self.applianceName).set(orch_return['summary']['num_bgp_peers']) #Setting Metric
        bgpSubsInstalled.labels(applianceName=self.applianceName).set(orch_return['summary']['num_subs_installed']) #Setting Metric
        bgpOspfRoutes.labels(applianceName=self.applianceName).set(orch_return['summary']['num_ospf_rtes']) #Setting Metric
        bgpIBGPRoutes.labels(applianceName=self.applianceName).set(orch_return['summary']['num_ibgp_rtes']) #Setting Metric
        bgpEBGPRoutes.labels(applianceName=self.applianceName).set(orch_return['summary']['num_ebgp_rtes']) #Setting Metric

        for neighbor in orch_return['neighbor']['neighborState']:
            peer_ip = neighbor['peer_ip']
            bgpNeighborState.labels(applianceName=self.applianceName, peer_ip=peer_ip).set(neighbor['peer_state']) #Setting Metric
            bgpNeighborStateStr.labels(applianceName=self.applianceName, peer_ip=peer_ip, asn=neighbor['asn']).info({'bgpNeighborStateStr' : neighbor['peer_state_str']}) #Setting Metric
            bgpNeighborUptime.labels(applianceName=self.applianceName, peer_ip=peer_ip).set(neighbor['time_established']) #Setting Metric
            bgpNeighborReceivedPrefix.labels(applianceName=self.applianceName, peer_ip=peer_ip).set(neighbor['rcvd_pfxs']) #Setting Metric
            bgpNeighborSentPrefix.labels(applianceName=self.applianceName, peer_ip=peer_ip).set(neighbor['sent_pfxs']) #Setting Metric

        return orch_return
    


class collectApplianceOSPF():
    def __init__(self):
        pass


class collectApplianceFlows():
    def __init__(self,url,ne_pk,applianceName,key,verify_ssl,interval,debug, Break):
        self.url = url
        self.ne_pk = ne_pk
        self.applianceName = applianceName
        self.key = key
        self.verify_ssl = verify_ssl
        self.interval = interval
        self.debug = debug
        self.Break = Break
        self.orch = Orchestrator(url=self.url, verify_ssl=self.verify_ssl, api_key=self.key )
        while True:
            # get a list of all the methods in this class
            # loops over the list starting at index 1 to bypass __init__
            # calls the method
            methodList = inspect.getmembers(self, predicate=inspect.ismethod)
            for m in range(1, len(methodList)):
                i = methodList[m][1]()

                logToFile().debug(message=dict({methodList[m][1].__name__ : i}), debug=self.debug) 
                confirmReturn(func=methodList[m][1].__name__ ,dictionary=i, debug=self.debug) 
                writeResult(func=list(i.keys())[0], result=i['result'], debug=self.debug)
            if self.Break == False:
                wait(self.interval)
            else:
                break

    @errorHandler
    def _getApplianceFlows(self):
        orch_return = self.orch.get_appliance_flows(self.ne_pk, uptime="term5m")

        activeTotalFlows.labels(applianceName=self.applianceName).set(orch_return['active']['total_flows']) #Setting Metric
        activeStaleFlows.labels(applianceName=self.applianceName).set(orch_return['active']['stale_flows']) #Setting Metric
        activeInconsistentFlows.labels(applianceName=self.applianceName).set(orch_return['active']['inconsistent_flows']) #Setting Metric
        activeFlowsWithIssues.labels(applianceName=self.applianceName).set(orch_return['active']['flows_with_issues']) #Setting Metric
        activeFlowsOptimized.labels(applianceName=self.applianceName).set(orch_return['active']['flows_optimized']) #Setting Metric
        activeFlowsWithIgnores.labels(applianceName=self.applianceName).set(orch_return['active']['flows_with_ignores']) #Setting Metric
        activeFlowsPassthrough.labels(applianceName=self.applianceName).set(orch_return['active']['flows_passthrough']) #Setting Metric
        activeFlowsManagement.labels(applianceName=self.applianceName).set(orch_return['active']['flows_management']) #Setting Metric
        activeFlowsAsymmetric.labels(applianceName=self.applianceName).set(orch_return['active']['flows_asymmetric']) #Setting Metric
        activeFlowsRouteDropped.labels(applianceName=self.applianceName).set(orch_return['active']['flows_route_dropped']) #Setting Metric
        activeFlowsFirewallDropped.labels(applianceName=self.applianceName).set(orch_return['active']['flows_firewall_dropped']) #Setting Metric

        inactiveTotalFlows.labels(applianceName=self.applianceName).set(orch_return['inactive']['total_flows']) #Setting Metric
        inactiveStaleFlows.labels(applianceName=self.applianceName).set(orch_return['inactive']['stale_flows']) #Setting Metric
        inactiveInconsistentFlows.labels(applianceName=self.applianceName).set(orch_return['inactive']['inconsistent_flows']) #Setting Metric
        inactiveFlowsWithIssues.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_with_issues']) #Setting Metric
        inactiveFlowsOptimized.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_optimized']) #Setting Metric
        inactiveFlowsWithIgnores.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_with_ignores']) #Setting Metric
        inactiveFlowsPassthrough.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_passthrough']) #Setting Metric
        inactiveFlowsManagement.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_management']) #Setting Metric
        inactiveFlowsAsymmetric.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_asymmetric']) #Setting Metric
        inactiveFlowsRouteDropped.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_route_dropped']) #Setting Metric
        inactiveFlowsFirewallDropped.labels(applianceName=self.applianceName).set(orch_return['inactive']['flows_firewall_dropped']) #Setting Metric

        return orch_return


import yaml
import os
from .logger import log,logToFile
from .args import parser
from .utls import createDebugFolder

def readConfig(filePath,debug):
    with open(filePath, "r") as stream:
        try:
            ymlVars = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            log().error(exc)
    if debug:
        logToFile().debug(message=ymlVars,debug=debug)
        log().debug(ymlVars)

    return ymlVars


class settings():
    @staticmethod
    def getEnvironmentVar(key,debug):
        try:
            out = os.environ[key]
            if len(out) == 0:
                out = False
        except KeyError as error:
            out = False
        if debug:
            log().debug({"environment": key, "value" : out})
            logToFile().debug(message={"environment": key, "value" : out},debug=debug)
        return out

    def __init__(self):
        self.userInput = parser()
# If the debug flag is passed then this function will create the folder and file to write the debug messages
        createDebugFolder(self.userInput.debug)

        checkENV = settings.getEnvironmentVar('SP_FILE_PATH',self.userInput.debug)
        if checkENV == False:
            filepath = self.userInput.file_path
        else:
            filepath = checkENV
        
        jsonConfigs = readConfig(filepath, self.userInput.debug)
        self.config = jsonConfigs




    def debug(self):
        out = {}

        try:
            checkENV = settings.getEnvironmentVar('SP_DEBUG',self.userInput.debug)
            if checkENV == False:
               out.update({"debug" :self.userInput.debug})
            else:
                out.update({"debug" : checkENV})

        except Exception as error:
         #   log().error(error)
            out.update({"debug" : False})

        return out



    def Break(self):
        out = {}

        try:
            checkENV = settings.getEnvironmentVar('SP_BREAK',self.userInput.debug)
            if checkENV == False:
                out.update({"Break" :self.userInput.Break})
            else:
                out.update({"Break" : checkENV})

        except Exception as error:
         #   log().error(error)
            out.update({"Break" : False})

        return out

    def port(self):
        out = {}

        try:
            checkENV = settings.getEnvironmentVar('SP_PORT',self.userInput.debug)
            if checkENV == False:
                out.update({"port" : self.config['config']['exporter']['port']})
            else:
                checkENV = int(checkENV)
                out.update({"port" : checkENV})

        except Exception as error:
         #   log().error(error)
            out.update({"port" :self.userInput.port})

        return out

    def orch(self):
        out = {}
        try:
            checkENV = settings.getEnvironmentVar('SP_ORCH_URL',self.userInput.debug)
            if checkENV == False:
                out.update({"url" : self.config['config']['silverpeak']['orch']})
            else:
                out.update({"url" : checkENV})
        except Exception as error:
            #log().error(error)
            out.update({"url": self.userInput.orch }) 
        try:
            checkENV = settings.getEnvironmentVar('SP_ORCH_KEY', False)
            if checkENV == False:
                out.update({"key": self.userInput.key})
            else:
                out.update({"key": checkENV})
        except Exception as error:
            #log().error(error)
            out.update({"key": self.userInput.key})
        try:
            checkENV = settings.getEnvironmentVar('SP_ORCH_SSL',self.userInput.debug)
            if checkENV == False:
                out.update({"verify_ssl" : self.config['config']['silverpeak']['verify_ssl']})
            else:
                checkENV = bool(checkENV)
                out.update({"verify_ssl" : checkENV})
        except Exception as error:
            #log().error(error)
            out.update({"verify_ssl" : False})

        return out
    
    def metricsOrchestrator(self):
        out = {}
        try:
            out.update({"collect" : self.config['orchestrator']['collect']})
        except Exception as error:
         #   log().error(error)
            out.update({"collect" : False})
        try:
            out.update({"interval" : self.config['orchestrator']['interval']})
        except Exception as error:
         #   log().error(error)
            out.update({"interval" : 60})

        return out

    def metricsAppliance(self):
        out = {}
        try:
            out.update({"appliances" : self.config['appliances']})
        except Exception as error:
         #   log().error(error)
            out.update({"appliances" : False})

        return out